<?php
require_once ASD_APPLICATION_PATH . '/models/AbstractModel.php';

/**
* Implements common functions of admin and metabox forms
*/
class Asd_Model_Amazon_FormHelper extends Asd_Model_AbstractModel
{
    /**
    * @var Asd_Model_Amazon_FormHelper
    */
    protected $_form;

    public function loadCategories($data=NULL)
    {
        $amazon = Asd_Application::getModel('Amazon');

        $hasCredentials = TRUE;
        $options = $this->_form->extractCredentials($data);
        if (is_null($options)) {
            $options = array();
            $optionsModel = Asd_Application::getModel('Options');

            foreach ($amazon->getCredentialFields() as $field) {
                $value = $optionsModel->getValue($field);
                if ('' == $value) {
                    $hasCredentials = FALSE;
                }
                $options[$field] = $value;
            }
        }

        foreach (Asd_Application::getModel('Amazon_Market')->getCodeList()
            as $marketCode
        ) {
            $options['market'] = $marketCode;

            $amazon->setOptions($options);

            $field = "{$marketCode}_category";
            $category = $this->_form->getElement($field);
            if (!$category) {
                // Market is not supported
                continue;
            }

            $categories = $amazon->getCategories();

            $category->setMultiOptions($categories);
            $category->setAttrib('disabled', FALSE);

            $parent = $this->_extractField($field, $data);

            if (!is_null($parent)) {
                if (isset($categories[$parent]) && $hasCredentials ) {
                    // Load subcategories
                    $subcategories = $amazon->getCategories($parent);

                    $subcategory = $this->_form->getElement("{$marketCode}_subcategory");
                    $subcategory->setMultiOptions($subcategories);
                    if ($subcategories) {
                        $subcategory->setAttrib('disabled', FALSE);
                    } else {
                        $subcategory->setValue(NULL);
                    }
                } else {
                    // Category not found or don't have credentials
                    $category->setValue(NULL);
                    $this->_form->getElement("{$marketCode}_subcategory")
                        ->setValue(NULL);
                }
            }
        }

        return $this;
    }

    /**
     * Check data
     */
    public function isValid($data)
    {
        $this->_form->loadCategories($data);

        return $this->_form->isValid($data);
    }

    //
    // Utility methods
    //

    protected function _extractField($field, $data=NULL)
    {
        if (is_null($data)) {
            return $this->_form->getValue($field);
        }
        if (isset($data[$field])) {
            return $data[$field];
        }

        return NULL;
    }

    //
    // Getters & setters
    //

    public function setForm($value)
    {
        $this->_form = $value;

        return $this;
    }

    public function getForm()
    {
        return $this->_form;
    }

    /**
    * Not a singleton
    */
    static public function isSingleton()
    {
        return FALSE;
    }
}