<?php
require_once ASD_APPLICATION_PATH . '/models/AbstractModel.php';

class Asd_Model_Block_AmazonProduct extends Asd_Model_AbstractModel
{
    protected $_options;
    protected $_data;

    public function __construct()
    {
        $this->_options = Asd_Application::getModel('Options');
    }

    public function setData($value)
    {
        $this->_data = $value;

        return $this;
    }

    public function getTitle()
    {
        return $this->_data['ItemAttributes']['Title'];
    }

    public function getImage($size = 'MediumImage')
    {
        if (isset($this->_data[$size])) {
            return $this->_data[$size];
        }
        if (isset($this->_data['ImageSets']['ImageSet'][0][$size])) {
            return $this->_data['ImageSets']['ImageSet'][0][$size];
        }
        if (isset($this->_data['ImageSets']['ImageSet'][$size])) {
            return $this->_data['ImageSets']['ImageSet'][$size];
        }
        if (isset($this->_data['Variations']['Item'][0][$size])) {
            return $this->_data['Variations']['Item'][0][$size];
        }
        if (isset($this->_data['Variations']['Item'][0]['ImageSets']['ImageSet'][$size])) {
            return $this->_data['Variations']['Item'][0]['ImageSets']['ImageSet'][$size];
        }
    }

    public function getUrl()
    {
        return $this->_filterUrl($this->_data['DetailPageURL']);
    }

    public function getAddToCartUrl()
    {
        $amazon = Asd_Application::getModel('Amazon');

        $params = array(
            'AssociateTag' => $amazon->getOption('affiliate_id'),
            'SubscriptionId' => $this->_options->getValue('access_key'),
            'ASIN.1' => $this->_data['ASIN'],
            'Quantity.1' => 1,
        );
        $trackingId = $this->_options->getValue('tracking_id');
        if ('' != $trackingId) {
            $params['t'] = $trackingId;
        }

        return 'http://' . $amazon->getMarketConfig()->domain . '/gp/aws/cart/add.html?'
            . http_build_query($params, '_', '&');
    }

    public function getReviewsUrl()
    {
        if (isset($this->_data['ItemLinks']['ItemLink'])) {
            $items = $this->_data['ItemLinks']['ItemLink'];
            if (!isset($items[0])) {
                // Single item
                $items = array($items);
            }

            foreach ($items as $link) {
                if ('All Customer Reviews' == $link['Description']) {
                    return $this->_filterUrl($link['URL']);
                }
            }
        }

        // Amazon.es does not return ItemLinks
        if (isset($this->_data['CustomerReviews']['IFrameURL'])
            && isset($this->_data['CustomerReviews']['HasReviews'])
            && 'true' == $this->_data['CustomerReviews']['HasReviews']
        ) {
            return $this->_filterUrl($this->_data['CustomerReviews']['IFrameURL']);
        }

        return NULL;
    }

    public function getPriceInfo()
    {
        $listPrice = NULL;
        $listPriceAmount = NULL;
        $offerPrice = NULL;
        $offerPriceAmount = NULL;

        if (isset($this->_data['ItemAttributes']['ListPrice'])) {
            $listPrice = $this->_data['ItemAttributes']['ListPrice']['FormattedPrice'];
            $listPriceAmount = $this->_data['ItemAttributes']['ListPrice']['Amount'];
        }

        if (isset($this->_data['Offers']['Offer']['OfferListing']['Price']['Amount']))  {
            $price = $this->_data['Offers']['Offer']['OfferListing']['Price'];

            if ($price['Amount'] >= $listPriceAmount) {
                $listPrice = $price['FormattedPrice'];
                $listPriceAmount = $price['Amount'];
            } else {
                $offerPrice = $price['FormattedPrice'];
                $offerPriceAmount = $price['Amount'];
            }
        } else if (isset($this->_data['OfferSummary']['LowestNewPrice']['Amount']))  {
            $price = $this->_data['OfferSummary']['LowestNewPrice'];

            if ($price['Amount'] >= $listPriceAmount) {
                $listPrice = $price['FormattedPrice'];
                $listPriceAmount = $price['Amount'];
            } else {
                $offerPrice = $price['FormattedPrice'];
                $offerPriceAmount = $price['Amount'];
            }
        }

        if (is_null($listPrice)
            && isset($this->_data['VariationSummary']['LowestSalePrice'])
        ) {
            $listPrice =  $this->_data['VariationSummary']['LowestSalePrice']['FormattedPrice'];
            $listPriceAmount = $this->_data['VariationSummary']['LowestSalePrice']['Amount'];
        }

        if (isset($this->_data['VariationSummary']['LowestPrice']['FormattedPrice'])
            && isset($this->_data['VariationSummary']['HighestPrice']['FormattedPrice'])
        ){
            $lowestPrice = $this->_data['VariationSummary']['LowestPrice']['FormattedPrice'];
            $highestPrice = $this->_data['VariationSummary']['HighestPrice']['FormattedPrice'];
            if (($lowestPrice != $highestPrice) && ('' != $highestPrice)) {
                $variationPrice =  $lowestPrice . ' - ' . $highestPrice;
            } else {
                $variationPrice = $lowestPrice;
            }

            if (is_null($listPrice)
                || $listPriceAmount < $this->_data['VariationSummary']['HighestPrice']['Amount']
            ) {
                $listPrice = $variationPrice;
                $listPriceAmount = NULL;
            } else {
                // Emulate offer
                $offerPrice = $variationPrice;
                $offerPriceAmount = NULL;
            }
        }

        $newMinPrice = NULL;
        $newOfferCount = NULL;
        if (isset($this->_data['OfferSummary']['TotalNew'])
            && $this->_data['OfferSummary']['TotalNew']
            && isset($this->_data['OfferSummary']['LowestNewPrice']['Amount'])
            && $this->_data['OfferSummary']['LowestNewPrice']['Amount'] < (!is_null($offerPriceAmount)? $offerPriceAmount : $listPriceAmount)
        ) {
            $newMinPrice = $this->_data['OfferSummary']['LowestNewPrice']['FormattedPrice'];
            $newOfferCount = (int)$this->_data['OfferSummary']['TotalNew'];
        }

        $usedMinPrice = NULL;
        $usedOfferCount = NULL;
        if (isset($this->_data['OfferSummary']['TotalUsed'])
            && $this->_data['OfferSummary']['TotalUsed']
            && isset($this->_data['OfferSummary']['LowestUsedPrice']['Amount'])
            && $this->_data['OfferSummary']['LowestUsedPrice']['Amount'] < (!is_null($offerPriceAmount)? $offerPriceAmount : $listPriceAmount)
        ) {
            $usedMinPrice = $this->_data['OfferSummary']['LowestUsedPrice']['FormattedPrice'];
            $usedOfferCount = (int)$this->_data['OfferSummary']['TotalUsed'];
        }

        $available = TRUE;
        if (isset($this->_data['OfferSummary'])
            && 0 == $this->_data['OfferSummary']['TotalNew']
            && 0 == $this->_data['OfferSummary']['TotalUsed']
            && 0 == $this->_data['OfferSummary']['TotalCollectible']
            && 0 == $this->_data['OfferSummary']['TotalRefurbished']
            && isset($this->_data['ItemAttributes']['ListPrice'])
        ) {
            $available = FALSE;
            // Check variations
            if (isset($this->_data['Variations']['Item'])) {
                $variations = isset($this->_data['Variations']['Item']['ASIN'])
                    ? array($this->_data['Variations']['Item'])
                    : $this->_data['Variations']['Item'];
                foreach ($variations as $variation) {
                    if (isset($variation['Offers']['Offer'])
                        || isset($variation['Offer'])
                    ) {
                        $available = TRUE;
                        break;
                    }
                }
            }
        }

        $result = array(
            'list_price' => $listPrice,
            'offer_price' => $offerPrice,
            'available' => $available,
            'offer_summary' => array(),
        );

        if (!is_null($newMinPrice)) {
            $result['offer_summary']['new'] = array(
                'min_price' => $newMinPrice,
                'count' => $newOfferCount,
            );
        }
        if (!is_null($usedMinPrice)) {
            $result['offer_summary']['used'] = array(
                'min_price' => $usedMinPrice,
                'count' => $usedOfferCount,
            );
        }

        return $result;
    }

    public function getDescription()
    {
        return isset($this->_data['etc']['description'])
            ? $this->_data['etc']['description']
            : NULL;
    }

    public function getRating()
    {
        return isset($this->_data['etc']['rating'])
            ? $this->_data['etc']['rating']
            : NULL;
    }

    public function getRatingCode()
    {
        if (!isset($this->_data['etc']['rating'])) {
            return NULL;
        }

        return str_replace('.', '_', sprintf('%01.1f',
            round($this->_data['etc']['rating']*2)/2));
    }

    protected function _filterUrl($url)
    {
        $trackingId = $this->_options->getValue('tracking_id');
        if ('' != $trackingId) {
            $url = add_query_arg('t', urlencode($trackingId), $url);
        }

        return $url;
    }
}